/*
 * PSquareGrid.java
 *
 * 2d rectangular grid of crypts.
 * von Neuman neighborhood
 * Note: wrapping not implemented correctly yet!!!
 * cyclic boundary on x axis only
 */

package sim;

import java.util.*;

public class PSquareGrid extends PGrid {
    private int _nXSize,_nYSize;
    private Boolean _bWrapX;  // cyclic boundary on X axis
    private Boolean _bWrapY;  // cyclic boundary on Y axis

    public PSquareGrid(int nXSize, int nYSize, CryptFactory factory, TissueEnvironmentSettings params, boolean bWrapX, boolean bWrapY) {
	super();
	_nXSize = nXSize;
	_nYSize = nYSize;
	_bWrapX = bWrapX;
	_bWrapY = bWrapY;
	_crypts = new Crypt[_nXSize*_nYSize];
	for (int i=0; i<_crypts.length; i++) {
	    _crypts[i] = factory.createCrypt();
	    _crypts[i].setGrid(this);
	    _crypts[i].setIndex(i);
	    _crypts[i].setParams(params);
	}
	for (int i=0; i<_crypts.length; i++) {
	    ArrayList n = getNeighbors(i);
	    for (int k=0; k<n.size(); k++) {
		Integer c = (Integer)n.get(k);
		_crypts[i].addNeighbor(_crypts[c.intValue()]);
	    }
	}
    }
    public double getX(int n) { return n%_nXSize; }
    public double getY(int n) { return n/_nXSize; }
    public double getMaxX() { return _nXSize-1; }
    public double getMaxY() { return _nYSize-1; }
    public boolean simulationIsComplete(long lCurTimestep) { return false; }

    /* getNeighbors
     * returns all grid cells touching crypt n
     */
    public ArrayList<Integer> getNeighbors(int n) {
	ArrayList<Integer> neighbors = new ArrayList<Integer>();
	int x = n%_nXSize;
	int y = n/_nXSize;
	for (int i=-1; i<=1; i++) {
	    for (int j=-1; j<=1; j++) {
		if ((i!=0 || j!=0) && j+y>=0 && j+y<_nYSize) {
		    //        if (i==0 ^ j==0) {
		    int a = (x+i+_nXSize)%_nXSize + (y+j)*_nXSize;
		    neighbors.add(new Integer(a));
		}
	    }
	}
	return neighbors;
    }

    public Crypt getCryptAtCenter() {
	return getCrypt(_nXSize/2 + (_nYSize/2)*_nXSize);
    }
}
