/*
 * TissueEnvironmentSettings.java
 */
package sim;

public class TissueEnvironmentSettings {
        // Simulation parameters
	private String _label; // the label associated with this set of parameters
	private int _maxSimulationYears = 50; // the number of years to run this simulation
        private int _gridSizeX = 100;  // size of tissue
        private int _gridSizeY = 100;  // size of tissue
	
        // Crypt parameters
        private double _meanCryptDivisionTime = 2.0; // average crypt division time in days
	private double _mutationRate = 0.0;          // mutation rate per day
	private double _wtBackgroundDeathRate = 0.00001; // crypt background death rate per day
	private double _mutantSurvivalAdvantageFactor = 0.5; // multiplier for _wtBackgroundDeathRate and _wtWoundImpactProbability, when <1 gives mutant survival adv.
	private double _mutantReproductiveAdvantageFactor=0.5; // multiplier for the mean number of days between cell divisions
	private double _wtWoundImpactProbability=0.01; // probability of death by a wounding event
	private double _mutantWoundResistanceFactor=0.5; // multiplier of probability of death from wounding, when <1 gives mutant resistance to wounding

        // Wounding parameters
	private double _woundIntervalTime=7;  // interval (in days) between wounding events
	private double _woundRadius=10;        // radius of wound
	private WoundingMode _woundingMode=WoundingMode.None; // whether wounding is (spatially) localized or distributed across the tissue
	public enum WoundingMode { None, Localized, Distributed }; // whether wounding is (spatially) localized or distributed across the tissue

	public String get_label() {
		return _label;
	}

        public int get_gridSizeX() { return _gridSizeX; }
        public int get_gridSizeY() { return _gridSizeY; }
        public void set_gridSizeX(int x) { _gridSizeX=x; }
        public void set_gridSizeY(int y) { _gridSizeY=y; }

	public double get_meanCryptDivisionTime() {
	    return _meanCryptDivisionTime;
	}

	public void set_meanCryptDivisionTime(
			double meanCryptDivisionTime) {
		_meanCryptDivisionTime = meanCryptDivisionTime;
	}

	public double get_mutantReproductiveAdvantageFactor() {
		return _mutantReproductiveAdvantageFactor;
	}

	public void set_mutantReproductiveAdvantageFactor(
			double reproductiveAdvantageFactor) {
		_mutantReproductiveAdvantageFactor = reproductiveAdvantageFactor;
	}

	public double get_mutantSurvivalAdvantageFactor() {
		return _mutantSurvivalAdvantageFactor;
	}

	public void set_mutantSurvivalAdvantageFactor(double survivalAdvantageFactor) {
		_mutantSurvivalAdvantageFactor = survivalAdvantageFactor;
	}

	public void set_mutantWoundResistanceFactor(double woundResistanceFactor) {
		_mutantWoundResistanceFactor = woundResistanceFactor;
	}

	public double get_mutantWoundResistanceFactor() {
		return _mutantWoundResistanceFactor;
	}

	public double get_mutationRate() {
		return _mutationRate;
	}

	public void set_mutationRate(double rate) {
		_mutationRate = rate;
	}

	public double get_wtBackgroundDeathRate() {
		return _wtBackgroundDeathRate;
	}

	public void set_wtBackgroundDeathRate(double backgroundDeathRate) {
		_wtBackgroundDeathRate = backgroundDeathRate;
	}

	public double get_wtWoundImpactProbability() {
		return _wtWoundImpactProbability;
	}

	public void set_wtWoundImpactProbability(double woundImpactProbability) {
		_wtWoundImpactProbability = woundImpactProbability;
	}
	
	public void set_label(String label) {
		_label = label;
	}

	public void set_woundIntervalTime(double woundIntervalTime) {
		_woundIntervalTime = woundIntervalTime;
	}

	public double get_woundIntervalTime() {
		return _woundIntervalTime;
	}
	
	public void set_woundRadius(double woundRadius) {
		_woundRadius = woundRadius;
	}

	public double get_woundRadius() {
		return _woundRadius;
	}
	
	public void set_maxSimulationYears(int maxyears) {
		_maxSimulationYears = maxyears;
	}
	
	public int get_maxSimulationYears() {
		return _maxSimulationYears;
	}

	public WoundingMode get_woundingMode() {
		return _woundingMode;
	}

	public void set_woundingMode(WoundingMode mode) {
		_woundingMode = mode;
	}
}
