/*
 * PGridView.java
 * Renders a PGrid in a Graphics environment
 */

package ui;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import sim.*;

public class PGridView extends JPanel 
                       implements MouseListener {
	
	private PGrid _grid;
	private int _selected;
	private int _nScale;
        private int _nCryptSize;
        private int _xoffset;
        private int _yoffset;
	
	public PGridView (PGrid grid) {
		addMouseListener(this);
		setBackground(Color.black);
		_grid = grid;
		_selected = 0;
	}
	
	public void paintComponent(Graphics g) {

		// jte -- this greatly improves the image quality
		((Graphics2D)g).setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );

		setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		super.paintComponent(g);
		FontMetrics fm = g.getFontMetrics();
		Dimension d = getSize();
		_nScale = (int)(d.getHeight()/(_grid.getMaxY()+2));
		int temp = (int)(d.getWidth()/(_grid.getMaxX()));
		if (temp<_nScale)
			_nScale = temp;
		_nCryptSize = (int)(_nScale/1.3);
		_xoffset = _nCryptSize;
		_yoffset = _nCryptSize;
                //                ArrayList<Integer> neighbors = _grid.getNeighbors(_selected);
		for (int i=0; i<_grid.getSize(); i++) {
			Crypt crypt = _grid.getCrypt(i);
			if (crypt!=null) {
				double x = crypt.getX();
				double y = crypt.getY();
				g.setColor(_grid.getCrypt(i).getColor());
				g.fillOval(_xoffset + (int)(x*_nScale),_yoffset + (int)(y*_nScale),
						_nCryptSize,_nCryptSize);
				/*				if (i==_selected) {   // highlight the selected crypt
					g.setColor(Color.white);
                                        g.drawOval(_xoffset + (int)(x*_nScale) - 2,_yoffset + (int)(y*_nScale)-2,
                                                   _nCryptSize+4,_nCryptSize+4);
						   } */
                                /*				
                                else {
                                  boolean bFound = false;
                                  for (int a=0; a<neighbors.size(); a++) {
                                    int nx = ((Integer)neighbors.get(a)).intValue();
                                    if (nx==i) {      // highlight the neighbors of the selected crypt 
                                      g.setColor(Color.blue);
                                        g.drawOval(_xoffset + (int)(x*_nScale) - 2,_yoffset + (int)(y*_nScale)-2,
                                                   _nCryptSize+4,_nCryptSize+4);
                                      break;
                                    }
                                  }
                                  }*/
				 
			}
		}
		
		// jte -- disabled crypt info display
/*
		Crypt selected = _grid.getCrypt(_selected);
		if (selected!=null) {
			g.setColor(selected.getColor());
			g.drawString(selected.getInfo(),0,((int)_grid.getMaxY()+1)*_nScale+10);
		}
*/
	}
	public void update() {repaint();}
	public void mouseClicked(MouseEvent e) {
		int mousex = e.getX();
		int mousey = e.getY();
		double closestdist = 100000.0;
		int closest = -1;
		for (int i=0; i<_grid.getSize(); i++) {
			Crypt crypt = _grid.getCrypt(i);
			if (crypt!=null) {
				double x = crypt.getX();
				double y = crypt.getY();

				double dist = (_xoffset+(int)(x*_nScale)+_nCryptSize/2-mousex)*(_xoffset+(int)(x*_nScale)+_nCryptSize/2-mousex) + 
                                  (_yoffset+(int)(y*_nScale)+_nCryptSize/2-mousey)*(_yoffset+(int)(y*_nScale)+_nCryptSize/2-mousey);
				if (dist<closestdist) {
					closestdist = dist;
					closest = i;
				}
			}
		}
		_selected = closest;
		repaint();
	}
	public void mousePressed(MouseEvent e) {
	}
	public void mouseReleased(MouseEvent e) {
	}
	public void mouseEntered(MouseEvent e) {
	}
	public void mouseExited(MouseEvent e) {
	}
}
